@@ -31,8 +31,8 @@
 # @author Paul Klemm
 # @email klemmp@staff.uni-marburg.de
 # @company Bioinformatics, University of Leipzig
-# @version 1
-# @date 8-04-2019
+# @version 2
+# @date 8-12-2020
 #
 ##########################################################################################
 
@@ -222,8 +222,9 @@ a!=c[b];c[b]=a;return d}};return p});
     <style>
 		table.greyGridTable {
 		  border: 2px solid #FFFFFF;
-		  width: 100%;
+/*		  width: 100%; */
 		  text-align: center;
+		  min-width:100%;
 		  border-collapse: collapse;
 		  table-layout: fixed;
 		  z-index: 999;
@@ -239,7 +240,6 @@ a!=c[b];c[b]=a;return d}};return p});
 		}
 		table.greyGridTable thead {
 		  background: #EBEBEB;
-		  border-bottom: 4px solid #333333;
 
 		  font-size: 15px;
 		  font-weight: bold;
@@ -264,8 +264,8 @@ a!=c[b];c[b]=a;return d}};return p});
 	      -o-transform: rotate(-0deg) translatex(-0%);
 	      transform: rotate(-0deg) translatex(-0%);
 	      font-weight: normal;
-		  word-wrap: break-word;         /* All browsers since IE 5.5+ */
-		    overflow-wrap: break-word;     /* Renamed property in CSS3 draft spec */
+/*		  word-wrap: break-word;         
+		    overflow-wrap: break-word;      */
 
 		}
 		table.greyGridTable thead th:nth-child(-n+2) {
@@ -282,6 +282,7 @@ a!=c[b];c[b]=a;return d}};return p});
 	      -ms-transform: rotate(-90deg) translatex(-0%);
 	      -o-transform: rotate(-90deg) translatex(-0%);
 	      transform: rotate(-90deg) translatex(-0%);
+
 		}
 
 		table.greyGridTable thead th:nth-child(-n+3) {
@@ -315,7 +316,6 @@ a!=c[b];c[b]=a;return d}};return p});
 		  font-size: 14px;
 		  font-weight: bold;
 		  color: #333333;
-		  border-top: 4px solid #333333;
 		}
 		table.greyGridTable tfoot td {
 		  font-size: 14px;
@@ -345,7 +345,8 @@ a!=c[b];c[b]=a;return d}};return p});
 		}
 		.clusterize-scroll{
 		    position: absolute;
-		    top: 135px;
+		    top: 155px;
+		    width: 100%;
 		    left: 0;
 		    right: 0;
 		    padding:0;
@@ -371,10 +372,13 @@ a!=c[b];c[b]=a;return d}};return p});
 		}
 		.overlay{
 		  padding:5px;
-		  background: #369;
+		  background: #f0f2ff;
 		  position:fixed;
 		  width:100%;
 		  height:100px;
+		  left:0px;
+		  border-top: 3px #dbdbdb solid;
+		  border-bottom: 3px #dbdbdb solid;
 		}
 		.left{
 			width: 40%;
@@ -383,13 +387,11 @@ a!=c[b];c[b]=a;return d}};return p});
 		    height:100%;
 		}
 		.middleA {    
-			background: #369;
 			width: 10%;
 		    height:90%;
 		    float: left;	
 		}
 		.middleB {
-		    background: #369;
 		    float: left;	
 		    height:90%;
 		    margin-left:5%;
@@ -1117,7 +1119,7 @@ print '
 		    		if(genes!="" && genes.slice(-1)!=","){
 		    			genes+=",";
 		    		}
-		    		genes+=tds[i].innerHTML.replace(/([<]([^>]+)[>])/ig,"").replace(/([(]([+-]+)[)])/ig,",").replace(/[^()]+[(]([^)]+)[)]/ig,"$1,").replace(/ /ig,"").replace(/,$/ig,"").replace(/,,+/ig,",");
+		    		genes+=tds[i].innerHTML.replace(/([<]([^>]+)[>])/ig,"").replace(/([(]([+-]+)[)])/ig,",").replace(/[^()]+[(]([^)]+)[)]/ig,"$1,").replace(/ /ig,"").replace(/,$/ig,"").replace(/,,+/ig,",").replace(/[()]/g,"");
 		    	}
 		    }
 
