@@ -28,8 +28,8 @@
 # @author Paul Klemm
 # @email klemmp@staff.uni-marburg.de
 # @company Bioinformatics, University of Leipzig
-# @version 3
-# @date 30-04-2020
+# @version 5
+# @date 1-29-2021
 #
 ##########################################################################################
 
@@ -53,6 +53,7 @@ proteinortho_grab_proteins.pl (options) QUERY FASTA1 (FASTA2 ...)
 		-exact        search patters are extended with a \b, that indicates end of word.
 		-source, -s   adds the filename (FASTA1,...) to the found gene-name
 		-F=s          char delimiter for multiple identifier if QUERY is a string input (default: ',')
+		-isoform      if you use proteinortho with --isoform option, then you need to set this option here too. 
 
 DESCRIPTION
  
@@ -100,6 +101,7 @@ ENDUSAGE
 my $query;
 my $help;
 my $tofiles=0;
+my $isoform=0;
 my $justid;
 my $prefix=">";
 my $doregex=0;
@@ -117,6 +119,7 @@ for(my $v = 0 ; $v < scalar @ARGV_copy ; $v++){
 	elsif($ARGV_copy[$v] =~ m/^--?(source|s)$/){$source=1;}
 	elsif($ARGV_copy[$v] =~ m/^--?F=(.*)$/){$del=$1;}
 	elsif($ARGV_copy[$v] =~ m/^--?E$/){$doregex=1;}
+	elsif($ARGV_copy[$v] =~ m/^--?isoform$/){$isoform=1;}
 	elsif($ARGV_copy[$v] =~ m/^--?exact$/){$exact=1;}
 	elsif($ARGV_copy[$v] =~ m/^-.+/){print $usage; print STDERR "ERROR: invalid option ".$ARGV_copy[$v]."!\n\n";exit(1);}
 	elsif(!defined($query)){$query = $ARGV_copy[$v];}
@@ -144,47 +147,67 @@ if($fail){
 my %qdata;
 # my $qdata_count = {};
 
-my $orthogroupcounter=0;
-my $genecounter=0;
+our $orthogroupcounter=0;
+our $genecounter=0;
+my $numOfFastas=0;
+my $line_i = 0;
+
+for(my $v = 0 ; $v < scalar @ARGV_copy ; $v++){
+	if($ARGV_copyiddone[$v]){next;}
+	$numOfFastas++;
+}
+our @filenames;
+
+my $foundHeader=0;
+
+# print STDERR (scalar keys %qdata)." vs ".($numOfFastas);
+
+sub processLine{
+	my $line = shift;
+	my $prefix = shift;
+
+	$line=~s/[\r\n]+$//; 
+
+	my @sp = split(/\t/,$line);
+	if(substr($line,0,1) eq "#"){$foundHeader=1;@filenames=@sp; next;}
+	if(scalar(@sp)>3){
+		for(my $v = 3 ; $v < scalar @sp ; $v++){
+			if($sp[$v] eq "*" || $sp[$v] eq ""){next;}
+			my @spp = split(",",$sp[$v]);
+
+			for(my $vv = 0 ; $vv < scalar @spp ; $vv++){
+
+				if($spp[$vv] eq "*" || $spp[$vv] eq ""){next;}
+				$spp[$vv]=~s/^\(//;$spp[$vv]=~s/\)$//;
+
+				if(!exists $filenames[$v]){
+					$filenames[$v]=""
+				}
+
+				$qdata{$filenames[$v]}{$spp[$vv]}=$prefix.".OrthoGroup".$orthogroupcounter;
+				$genecounter++;
+			}
+		}
+	}
+	$orthogroupcounter++;
+}
 
 unless(open(my $FH,'<',$query)) {
 	if($query eq "-"){
-		foreach my $line (<STDIN>) 
-		{
-			$line=~s/[\r\n]+$//;
-
-			if(length($line)==0 || $line eq ""){next;}
-
-			my @sp = split(/\t/,$line);
-			if(substr($line,0,1) eq "#"){@filenames=@sp; next;}
-			if(scalar(@sp)>3){
-				for(my $v = 3 ; $v < scalar @sp ; $v++){
-					if($sp[$v] eq "*" || $sp[$v] eq ""){next;}
-					my @spp = split(",",$sp[$v]);
-					#if($exact){$sp[$v]='\b'.$sp[$v].'\b';}
-					for(my $vv = 0 ; $vv < scalar @spp ; $vv++){
-						#if(!$doregex && !$exact){$spp[$vv]=quotemeta($spp[$vv]);}
-						if($spp[$vv] eq "*" || $spp[$vv] eq ""){next;}
-
-						$qdata{"STDIN"}{$spp[$vv]}="STDIN.OrthoGroup".$orthogroupcounter;
-						$genecounter++;
-					}
-				}
-			}
-			$orthogroupcounter++;
-		}
+		foreach my $line (<STDIN>) { &processLine($line, "STDIN") }
 	}else{
 		my @sp = split($del,$query); 
 		for(my $v = 0 ; $v < scalar @sp ; $v++){
-			#if(!$doregex){$sp[$v]=quotemeta($sp[$v]);}
 			if(length($sp[$v])==0 || $sp[$v] eq ""){next;}
-			#if($exact){$sp[$v]='\b'.$sp[$v].'\b';}
+			$sp[$v]=~s/^\(//;$sp[$v]=~s/\)$//;
+
 			$qdata{"STDIN"}{$sp[$v]}=$sp[$v];
 			$genecounter++;
 		}
 	}
 }else{
-	if(!$exact){print STDERR "[STDERR] WARNING The -exact option is mandatory if a proteinortho file is given. -exact is now set.\n";$exact=1;}
+	if($isoform && $exact){print STDERR "[STDERR] WARNING The -isoform option is not compatible with -exact if a proteinortho file is given. -exact is now unset.\n";$exact=0;}
+	elsif(!$isoform && !$exact){print STDERR "[STDERR] WARNING The -exact option is mandatory if a proteinortho file is given. -exact is now set.\n";$exact=1;}
 	if($doregex){print STDERR "[STDERR] WARNING The -E option is not allowed if a proteinortho file is given. -E is now unset.\n";$doregex=0;}
 
 	my $query_basename=$query;
@@ -192,30 +215,17 @@ unless(open(my $FH,'<',$query)) {
 		$query_basename=$1;
 	}
 
-	@filenames;
-	while(<$FH>){
-		$_=~s/[\r\n]+$//;
-		my @sp = split(/\t/,$_);
-		if(substr($_,0,1) eq "#"){@filenames=@sp; next;}
-		if(scalar(@sp)>3){
-			for(my $v = 3 ; $v < scalar @sp ; $v++){
-				if($sp[$v] eq "*" || $sp[$v] eq ""){next;}
-				my @spp = split(",",$sp[$v]);
-				#if($exact){$sp[$v]='\b'.$sp[$v].'\b';}
-				for(my $vv = 0 ; $vv < scalar @spp ; $vv++){
-					#if(!$doregex && !$exact){$spp[$vv]=quotemeta($spp[$vv]);}
-					if($spp[$vv] eq "*" || $spp[$vv] eq ""){next;}
-					$qdata{$filenames[$v]}{$spp[$vv]}=$query_basename.".OrthoGroup".$orthogroupcounter;
-					$genecounter++;
-				}
-			}
-		}
-		$orthogroupcounter++;
-	}
+	while(<$FH>){ &processLine($_, $query_basename) }
 	close($FH);
 	print STDERR "[STDERR] Done reading the query $query file. Now I know $orthogroupcounter groups with $genecounter genes/proteins in total.\n";
 }
 
+
+if( $foundHeader==0 && $numOfFastas > 3 && $genecounter > 20){
+	print STDERR "\nWARNING : The header of the proteinortho file is missing, this can increase the runtime dramatically. Please include the first line (starting with '#'), to accelerate this program.\n$NC\n";
+	sleep 1;
+}
+
 if( $tofiles==1 && ($orthogroupcounter > 100) ){
 	print STDERR "\n!!!\nWARNING : This call will produce $orthogroupcounter files (one for each orthology group) !\nIn the *.html file you can individually extract single groups by clicking on the front part of a row.\n$NC";
 	print STDERR "Press 'strg+c' to prevent me from proceeding or wait 20 seconds to continue...\n!!!\n";
@@ -229,20 +239,16 @@ my $cur_gene_filename="";
 my %cur_gene_firsttime;
 my $genecounterfound=0;
 my $basename = "";
-my $numOfFastas=0;
-
-for(my $v = 0 ; $v < scalar @ARGV_copy ; $v++){
-	if($ARGV_copyiddone[$v]){next;}
-	$numOfFastas++;
-}
 
 my $fastai=1;
 
+my %foundIDs;
+
 for(my $v = 0 ; $v < scalar @ARGV_copy ; $v++){
 
 	if($ARGV_copyiddone[$v]){next;}
 
-	if($tofiles){print STDERR "[STDERR] ($fastai/$numOfFastas) : ";if($basename ne ""){print STDERR "Done reading $basename. "}print STDERR "Start reading the fasta file ".($ARGV_copy[$v])."\n";}
+	print STDERR "[STDERR] ($fastai/$numOfFastas) : ";if($basename ne ""){print STDERR "Done reading $basename. "}print STDERR "Start reading the fasta file ".($ARGV_copy[$v])."\n";
 	$fastai++;
 
 	$basename = $ARGV_copy[$v];
@@ -294,6 +300,8 @@ for(my $v = 0 ; $v < scalar @ARGV_copy ; $v++){
 					$cur_gene_filename=$qdata{$basename}{$genename}.".fasta";
 					$geneprintswitch = 1;
 					$genecounterfound++;
+
+					delete $qdata{$basename}{$genename};
 				}
 
 			}else{ # fallback, if the basename (filename) does not exists, try all 
@@ -302,20 +310,41 @@ for(my $v = 0 ; $v < scalar @ARGV_copy ; $v++){
 					if($geneprintswitch){last;}
 					foreach my $key (keys %{$qdata{$filename}}) { 
 						my $regexv=$key; 
+						my $curLine_test = $curLine;
 						
-						if(!$doregex){$regexv=quotemeta($regexv);}
-						if($exact){$regexv='\b'.$regexv.'\b';}
+						if(!$doregex && !$exact){$regexv=quotemeta($regexv);}
+
+						my $test_match = 0;
+						if( !$exact ){
+							
+							# use regular expression
+							$test_match = $curLine_test =~ $regexv;
+
+						}else{
+							
+							# directly compare starting with first 5 character of fasta entry as offset
+							my $offset = 1; # start at 1 -> fasta entries starts with ">"
+							while( !( $test_match = substr($curLine,$offset,length $key) eq $key ) && $offset < 5 ){
+								$offset++;
+							} 
+						}
+
+						if( $test_match ){
+
+							if($qdata{$filename}{$key} eq ""){
+								print STDERR "[STDERR] WARNING The input ($key) was found multiple times in the fasta files ".(!$exact ? "(maybe try --exact)." : ".")."\n";
+							}
 
-						if( $curLine =~ $regexv ){
-									
 							my $headerstr=$curLine;
 							if($source){$headerstr=$headerstr." ".$basename;}
 
 							$cur_gene.=$headerstr."\n";	
-							$cur_gene_filename=$qdata{$filename}{$key}.".fasta";
+							$cur_gene_filename = $qdata{$filename}{$key}.".fasta";
 							$geneprintswitch = 1;
 							$genecounterfound++;
 
+							delete $qdata{$filename}{$key};
+
 							last;
 						}
 					}
@@ -349,6 +378,20 @@ if($genecounter != $genecounterfound){
 	print STDERR "[STDERR] WARNING The input ($query) contains $genecounter queries, but I extracted $genecounterfound entries out of the fasta(s).";
 	if(!$exact){print STDERR " If this is not desired, please consider using the -exact option";}elsif($genecounter > $genecounterfound){print STDERR "\n-> This should not have happen, maybe some fasta files are missing as input?\n(If you cannot solve this error, please send a report to incoming+paulklemm-phd-proteinortho-7278443-issue-\@incoming.gitlab.com or visit https://gitlab.com/paulklemm_PHD/proteinortho/wikis/Error%20Codes for more help. Further more all mails to lechner\@staff.uni-marburg.de are welcome)\n";}
 	print STDERR "\n";
+
+	if($genecounterfound < $genecounter){
+		print STDERR "The following ids were not found:\n";
+		my $counter=0;
+		foreach my $filename (keys %qdata) { 
+			foreach my $key (keys %{$qdata{$filename}}) { 
+				if(10 < $counter++){last}
+				print STDERR $key."\n"; 
+			}
+			if(10 < $counter){print STDERR " ...\n";last}
+		}
+		print STDERR "\nPlease make sure that the upper ids are part of the given fasta files (try searching these in the given fasta files) !\n";
+	}
+	
 }else{
 	print STDERR "[STDERR] All entries of the query are found in the fasta(s).\n";
-}
\ No newline at end of file
+}
